﻿using System.Linq;
using BAGA.Repository.Repositories;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using BAGA;
using System.Collections.Generic;


namespace POCOEFTests
{
    
    
    /// <summary>
    ///This is a test class for ReservationRepositoryTest and is intended
    ///to contain all ReservationRepositoryTest Unit Tests
    ///</summary>
  [TestClass()]
  public class ReservationRepositoryTest
  {


    private TestContext testContextInstance;

    /// <summary>
    ///Gets or sets the test context which provides
    ///information about and functionality for the current test run.
    ///</summary>
    public TestContext TestContext
    {
      get
      {
        return testContextInstance;
      }
      set
      {
        testContextInstance = value;
      }
    }

    


[TestMethod()]
[ExpectedException(typeof(ArgumentOutOfRangeException),
                      "GetReservations for Customer allowed an ID<1")]
public void Passing_ID_of_Zero_To_GetReservationsForCustomer_Throws()
{
 
    var context = new FakeContext();
    var repository = new ReservationRepository(context);
  int invalidCustomerId = 0; //invalid id, should be >0
  repository.GetReservationsForCustomer(invalidCustomerId);
}

[TestMethod()]
public void Passing_ID_Greater_Than_Zero_To_GetReservationsForCustomer_doesntthrow()
{
 var context = new FakeContext();
  var target = new ReservationRepository(context);
  int customerId = 20; //valid id
  IList<Reservation> actual = target.GetReservationsForCustomer(customerId);
  Assert.IsNotNull(actual);
}
 
  [TestMethod()]
[DeploymentItem("Repositories.dll")]
public void GetReservationsForCustomerValidIdUnitOfWork()
{
  var context = new FakeContext();
    var uow = new UnitOfWork(context);
  var target = new ReservationRepository(uow);
  int customerId = 20; //valid id
  IList<Reservation> actual = target.GetReservationsForCustomer(customerId);
  Assert.IsNotNull(actual);
}

  [TestMethod()]
  public void IncludeTest()
  {
    var context = new FakeContext();
    var contact = context.Reservations.FirstOrDefault(); //.Include("Payments").FirstOrDefault();
    Assert.IsNotNull(contact.Payments);
  }
[TestMethod()]
public void PaymentStatus_Returns_UnPaid_When_There_Are_No_Payments()
{
  //create a new reservation, new trip, new list of Payments
  var reservation = new Reservation();
  var trip = new Trip { TripCostUSD = 1000 };
  //bind the trip and payments to the reservation
  reservation.Trip = trip;

  Assert.AreEqual(reservation.GetPaymentStatus(),Reservation.PaymentStatus.UnPaid);
}
    [TestMethod()]
public void PaymentStatus_Returns_PartiallyPaid_When_There_Are_Insufficient_Payments()
{
  var reservation = new Reservation();
  reservation.Trip = new Trip { TripCostUSD = 1000 };
  reservation.Payments.Add(new Payment { Amount = 500 });
  Assert.AreEqual(reservation.GetPaymentStatus(),
                  Reservation.PaymentStatus.PartiallyPaid);

}

  }
}
 